"""
Alternate namespece for cytoolz such that all functions are curried

Currying provides implicit partial evaluation of all functions

Example:

    Get usually requires two arguments, an index and a collection
    >>> from cytoolz.curried import get
    >>> get(0, ('a', 'b'))
    'a'

    When we use it in higher order functions we often want to pass a partially
    evaluated form
    >>> data = [(1, 2), (11, 22), (111, 222)]
    >>> list(map(lambda seq: get(0, seq), data))
    [1, 11, 111]

    The curried version allows simple expression of partial evaluation
    >>> list(map(get(0), data))
    [1, 11, 111]

See Also:
    cytoolz.functoolz.curry
"""

import cytoolz
from cytoolz import *
from cytoolz.curried_exceptions import *


# Here is the recipe used to create the list below
# (and "cytoolz/tests/test_curried_toolzlike.py" verifies the list is correct):
#
# import toolz
# import toolz.curried
#
# for item in sorted(key for key, val in toolz.curried.__dict__.items()
#                    if isinstance(val, toolz.curry)):
#      print '%s = cytoolz.curry(%s)' % (item, item)

accumulate = cytoolz.curry(accumulate)
assoc = cytoolz.curry(assoc)
cons = cytoolz.curry(cons)
countby = cytoolz.curry(countby)
dissoc = cytoolz.curry(dissoc)
do = cytoolz.curry(do)
drop = cytoolz.curry(drop)
filter = cytoolz.curry(filter)
get = cytoolz.curry(get)
get_in = cytoolz.curry(get_in)
groupby = cytoolz.curry(groupby)
interleave = cytoolz.curry(interleave)
interpose = cytoolz.curry(interpose)
itemfilter = cytoolz.curry(itemfilter)
itemmap = cytoolz.curry(itemmap)
iterate = cytoolz.curry(iterate)
join = cytoolz.curry(join)
keyfilter = cytoolz.curry(keyfilter)
keymap = cytoolz.curry(keymap)
map = cytoolz.curry(map)
mapcat = cytoolz.curry(mapcat)
memoize = cytoolz.curry(memoize)
merge_with = cytoolz.curry(merge_with)
nth = cytoolz.curry(nth)
partition = cytoolz.curry(partition)
partition_all = cytoolz.curry(partition_all)
partitionby = cytoolz.curry(partitionby)
pluck = cytoolz.curry(pluck)
reduce = cytoolz.curry(reduce)
reduceby = cytoolz.curry(reduceby)
remove = cytoolz.curry(remove)
sliding_window = cytoolz.curry(sliding_window)
sorted = cytoolz.curry(sorted)
tail = cytoolz.curry(tail)
take = cytoolz.curry(take)
take_nth = cytoolz.curry(take_nth)
unique = cytoolz.curry(unique)
update_in = cytoolz.curry(update_in)
valfilter = cytoolz.curry(valfilter)
valmap = cytoolz.curry(valmap)



from .itertoolz import *

from .functoolz import *

from .dicttoolz import *

from .recipes import *

from .compatibility import map, filter

# from . import sandbox

from functools import partial, reduce

sorted = sorted

# Aliases
comp = compose

from ._version import __version__, __toolz_version__



""" Build ``cytoolz`` with or without Cython.

Deployed versions of CyToolz do not rely on Cython by default even if the
user has Cython installed.  A C compiler is used to compile the distributed
*.c files instead.

Pass "--cython" or "--with-cython" as a command line argument to setup.py
to build the project using Cython.

Pass "--no-cython" or "--without-cython" to disable usage of Cython.

For convenience, developmental versions (with 'dev' in the version number)
automatically use Cython unless disabled via a command line argument.

"""
import os.path
import sys
from distutils.core import setup
from distutils.extension import Extension

info = {}
filename = os.path.join('cytoolz', '_version.py')
exec(compile(open(filename, "rb").read(), filename, 'exec'), info)
VERSION = info['__version__']

try:
    from Cython.Build import cythonize
    has_cython = True
except ImportError:
    has_cython = False

is_dev = 'dev' in VERSION
use_cython = is_dev or '--cython' in sys.argv or '--with-cython' in sys.argv
if '--no-cython' in sys.argv:
    use_cython = False
    sys.argv.remove('--no-cython')
if '--without-cython' in sys.argv:
    use_cython = False
    sys.argv.remove('--without-cython')
if '--cython' in sys.argv:
    sys.argv.remove('--cython')
if '--with-cython' in sys.argv:
    sys.argv.remove('--with-cython')

if use_cython and not has_cython:
    if is_dev:
        raise RuntimeError('Cython required to build dev version of cytoolz.')
    print('WARNING: Cython not installed.  Building without Cython.')
    use_cython = False

if use_cython:
    suffix = '.pyx'
else:
    suffix = '.c'

ext_modules = []
for modname in ['dicttoolz', 'functoolz', 'itertoolz',
                'curried_exceptions', 'recipes', 'utils']:
    ext_modules.append(Extension('cytoolz.' + modname,
                                 ['cytoolz/' + modname + suffix]))

if use_cython:
    ext_modules = cythonize(ext_modules)


if __name__ == '__main__':
    setup(
        name='cytoolz',
        version=VERSION,
        description=('Cython implementation of Toolz: '
                     'High performance functional utilities'),
        ext_modules=ext_modules,
        long_description=(open('README.rst').read()
                          if os.path.exists('README.rst')
                          else ''),
        url='https://github.com/pytoolz/cytoolz',
        author='https://raw.github.com/pytoolz/cytoolz/master/AUTHORS.md',
        author_email='erik.n.welch@gmail.com',
        maintainer='Erik Welch',
        maintainer_email='erik.n.welch@gmail.com',
        license = 'BSD',
        packages=['cytoolz'],
        package_data={'cytoolz': ['*.pxd']},
        # include_package_data = True,
        keywords=('functional utility itertools functools iterator generator '
                  'curry memoize lazy streaming bigdata cython toolz cytoolz'),
        classifiers = [
            'Development Status :: 4 - Beta',
            'Intended Audience :: Developers',
            'Intended Audience :: Education',
            'Intended Audience :: Science/Research',
            'License :: OSI Approved :: BSD License',
            'Operating System :: OS Independent',
            'Programming Language :: Cython',
            'Programming Language :: Python',
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 2.6',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.2',
            'Programming Language :: Python :: 3.3',
            'Programming Language :: Python :: 3.4',
            'Topic :: Scientific/Engineering',
            'Topic :: Scientific/Engineering :: Information Analysis',
            'Topic :: Software Development',
            'Topic :: Software Development :: Libraries',
            'Topic :: Software Development :: Libraries :: Python Modules',
            'Topic :: Utilities',
        ],
        # zip_safe=False
    )
